package com.divillysausages.connected.logic 
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.math.MathHelper;
	import com.greensock.easing.Back;
	import com.greensock.TweenLite;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Shape;
	import flash.display.Sprite;
	
	/**
	 * the picture of the guy that's talking to us
	 * @author Damian Connolly
	 */
	public class Pic extends Sprite implements IUpdateObj
	{
		
		/*******************************************************************************************/
		
		private var m_image:gRobot 			= null;
		private var m_imageGlitch:Bitmap	= null;
		private var m_imageGlitchMask:Shape	= null;
		private var m_isGlitching:Boolean	= false;// are we currently glitching?
		private var m_currTime:Number		= 0.0;	// curr wait time
		private var m_waitTime:Number		= 3.0;	// the wait time before we glitch
		private var m_speed:Number			= 20.0; // speed for the glitch
		private var m_width:Number			= 0.0;	// original uncorrupted width
		private var m_height:Number			= 0.0;	// original uncorrupted height
		
		/*******************************************************************************************/
		
		public function get active():Boolean { return this.parent != null; }
		
		override public function get width():Number { return this.m_width; }
		override public function get height():Number { return this.m_height; }
		
		/*******************************************************************************************/
		
		public function Pic() 
		{
			var w:Number 	= 50.0;		// bg width
			var h:Number 	= 50.0;		// bg height
			var iw:Number	= w - 30.0;	// image width
			var ih:Number	= h - 20.0;	// image height
			this.m_width	= w;
			this.m_height	= h;
			
			// bg
			this.graphics.lineStyle( 1.0, 0xffffff );
			this.graphics.beginFill( 0xffffff, 0.5 );
			this.graphics.drawRect( 0.0, 0.0, w, h );
			this.graphics.endFill();
			
			// image
			this.m_image 	= new gRobot;
			this.m_image.x 	= w * 0.5 - this.m_image.width * 0.5;
			this.m_image.y 	= h - this.m_image.height;
			this.addChild( this.m_image );
			
			// image glitch
			var bmd:BitmapData = new BitmapData( this.m_image.width, this.m_image.height, true, 0x00000000 );
			bmd.draw( this.m_image );
			this.m_imageGlitch			= new Bitmap( bmd );
			this.m_imageGlitch.scaleX	= 1.05;
			this.m_imageGlitch.scaleY	= 1.05;
			this.m_imageGlitch.x 		= w * 0.5 - this.m_imageGlitch.width * 0.5;
			this.m_imageGlitch.y 		= h * 0.5 - this.m_imageGlitch.height * 0.5;
			this.addChild( this.m_imageGlitch );
			
			// image mask
			this.m_imageGlitchMask = new Shape;
			this.m_imageGlitchMask.graphics.beginFill( 0xff0000 );
			this.m_imageGlitchMask.graphics.drawRect( 0.0, 0.0, w, h * 0.1 );
			this.m_imageGlitchMask.graphics.endFill();
			this.addChild( this.m_imageGlitchMask );
			this.m_imageGlitch.mask 	= this.m_imageGlitchMask;
			this.m_imageGlitchMask.y	= -this.m_imageGlitchMask.height;
			
			// add to update
			DS.addToUpdate( this );
		}
		
		public function destroy():void
		{
			DS.removeFromUpdate( this );
			this.removeChildren( 0, int.MAX_VALUE );
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			this.m_image			= null;
			this.m_imageGlitch		= null;
			this.m_imageGlitchMask	= null;
		}
		
		public function show( x:Number, y:Number ):void
		{
			this.x		= -this.width;
			this.y		= y;
			TweenLite.to( this, 0.5, { x:x, ease:Back.easeOut } );
		}
		
		public function update( dt:Number ):void
		{
			// wait a bit before glitching
			if ( !this.m_isGlitching )
			{
				this.m_currTime += dt;
				if ( this.m_currTime >= this.m_waitTime )
				{
					this.m_currTime 	= 0.0;
					this.m_isGlitching 	= true;
				}
			}
			else
			{
				// make the image glitch wave a bit from side to side
				this.m_imageGlitch.x 		= this.m_width * 0.5 - this.m_imageGlitch.width * 0.5 + Math.sin( MathHelper.degreesToRadians( this.m_imageGlitchMask.y ) ) * 2.0;
				this.m_imageGlitchMask.y 	+= this.m_speed * dt;
				if ( this.m_imageGlitchMask.y >= this.m_height )
				{
					this.m_imageGlitchMask.y 	= -this.m_imageGlitchMask.height;
					this.m_isGlitching			= false;
				}
			}
		}
		
	}

}