package com.divillysausages.connected.logic 
{
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.math.MathHelper;
	import com.divillysausages.ds.util.Colour;
	import flash.display.BitmapData;
	import flash.display.Shape;
	import flash.filters.DisplacementMapFilter;
	import flash.filters.DisplacementMapFilterMode;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import org.osflash.signals.Signal;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class Planet extends SpaceItem
	{
		
		/*******************************************************************************************/
		
		private static const M_TEXTURE_SIZE:Number	= 376.0;
		private static const M_DISPLACE_SIZE:Number = 60.0;
		
		/*******************************************************************************************/
		
		/**
		 * The signal dispatched when we capture the planet; it should take one parameter of type Planet
		 */
		public var signalOnCapture:Signal = null;
		
		/**
		 * The signal dispatched when we miss the planet; it should take one parmeter of type Planet and
		 * one of type Number (dist missed by)
		 */
		public var signalOnMiss:Signal = null;
		
		/**
		 * The signal dispatched when we hit the planet; it should take one parameter of type Planet
		 */
		public var signalOnHit:Signal = null;
		
		/*******************************************************************************************/
		
		private var m_planetMask:Shape			= null;	// the planet itself
		private var m_texture:gTexture			= null;	// our planet texture
		private var m_displace:gDisplace		= null;	// the displace texture used for our filter
		private var m_captureOutline:Shape		= null;	// our catcher
		private var m_radius:Number				= 0.0;	// our radius
		private var m_captureSuppDist:Number	= 0.0;	// the distance for the capture support
		private var m_captureRotDir:int			= 0;	// the dir of our rotation
		private var m_captureRotSpeed:Number	= 0.0;	// our rotation speed
		private var m_captureRadius:Number		= 0.0;	// the capture radius for the planet
		private var m_texRotDirX:int			= 0;	// the rotation dir for the texture (x)
		private var m_texRotDirY:int			= 0;	// the rotation dir for the texture (y)
		private var m_texRotSpeedX:Number		= 0.0;	// the rotation speed for the texture (x)
		private var m_texRotSpeedY:Number		= 0.0;	// the rotation speed for the texture (y)
		
		/*******************************************************************************************/
		
		override public function set scale( n:Number ):void
		{
			var oldScale:Number = this.scaleX;
			super.scale 		= n;
			//if( this.scaleX != oldScale ) // NOTE: remove this as it's killing the fps when the planet is big
				//this._createFilter(); // our scale has changed - recreate the filter
		}
		
		/*******************************************************************************************/
		
		public function Planet( ship:Ship ) 
		{			
			// set our properties
			this.m_planetMask		= new Shape;
			this.m_captureOutline	= new Shape;
			this.m_captureRotDir	= ( Math.random() < 0.5 ) ? -1 : 1;
			this.m_captureRotSpeed	= 40.0;
			this.m_texRotDirX		= ( Math.random() < 0.5 ) ? -1 : 1;
			this.m_texRotDirY		= ( Math.random() < 0.5 ) ? -1 : 1;
			this.m_texRotSpeedX	 	= 5.0 + Math.random() * 45.0;
			this.m_texRotSpeedY		= 5.0 + Math.random() * 45.0;
			this.m_radius			= 60.0;
			this.m_captureRadius	= 15.0;
			this.m_captureSuppDist	= 5.0;
			
			// create our planet mask
			this.m_planetMask.graphics.beginFill( 0xff0000 );
			this.m_planetMask.graphics.drawCircle( 0.0, 0.0, this.m_radius );
			this.m_planetMask.graphics.endFill();
			this.addChild( this.m_planetMask );
			
			// create our texture
			this.m_texture 		= new gTexture;
			this.m_texture.mask	= this.m_planetMask;
			this.addChild( this.m_texture );
			
			// create our display texture
			this.m_displace 		= new gDisplace;
			var scale:Number		= ( this.m_radius * 2.0 ) / Planet.M_DISPLACE_SIZE;
			this.m_displace.scaleX	= scale;
			this.m_displace.scaleY	= scale;
			this.m_displace.visible	= false;
			this.addChild( this.m_displace );
			
			// create our catcher outline
			this.m_captureOutline.graphics.lineStyle( 2.0, 0xffffff );
			this.m_captureOutline.graphics.drawCircle( 0.0, 0.0, this.m_radius );
			this.m_captureOutline.graphics.moveTo( this.m_radius, 0.0 );
			this.m_captureOutline.graphics.lineTo( this.m_radius + this.m_captureSuppDist, 0.0 );
			this.m_captureOutline.graphics.drawCircle( this.m_radius + this.m_captureSuppDist + this.m_captureRadius, 0.0, this.m_captureRadius );
			this.addChild( this.m_captureOutline );
			
			// position
			this.x = Math.random() * DS.stage.stageWidth * 0.5;
			this.y = Math.random() * DS.stage.stageHeight * 0.5;
			
			// create our signals
			this.signalOnCapture	= new Signal( Planet );			// planet captured
			this.signalOnMiss		= new Signal( Planet, Number );	// planet missed, dist
			this.signalOnHit		= new Signal( Planet );			// planet hit
			
			super( ship );
		}
		
		override public function destroy():void 
		{
			super.destroy();
			
			// clear our signals
			this.signalOnCapture.removeAll();
			this.signalOnMiss.removeAll();
			this.signalOnHit.removeAll();
				
			// null our references
			this.signalOnCapture	= null;
			this.signalOnMiss		= null;
			this.signalOnHit		= null;
			this.m_planetMask		= null;
			this.m_texture			= null;
			this.m_displace			= null;
			this.m_captureOutline	= null;
		}
		
		override public function reset():void 
		{
			super.reset();
			
			this.m_captureRotDir	= ( Math.random() < 0.5 ) ? -1 : 1;
			this.m_captureRotSpeed	= 30.0 + Math.random() * 20.0;
			this.m_texRotDirX		= ( Math.random() < 0.5 ) ? -1 : 1;
			this.m_texRotDirY		= ( Math.random() < 0.5 ) ? -1 : 1;
		}
		
		override public function update( dt:Number ):void
		{				
			super.update( dt );
			
			// update our rotation
			this.rotation += this.m_captureRotDir * this.m_captureRotSpeed * dt;
			
			// move the texture and clamp it
			this.m_texture.x			+= this.m_texRotSpeedX * this.m_texRotDirX * dt;
			this.m_texture.y 			+= this.m_texRotSpeedY * this.m_texRotDirY * dt;
			var halfTexSize:Number 		= Planet.M_TEXTURE_SIZE * 0.5;
			var halfTexSizeRad:Number	= halfTexSize - this.m_radius;
			if ( this.m_texture.x > halfTexSizeRad )
				this.m_texture.x -= halfTexSize;
			if ( this.m_texture.x < -halfTexSizeRad )
				this.m_texture.x += halfTexSize;
			if ( this.m_texture.y > halfTexSizeRad )
				this.m_texture.y -= halfTexSize;
			if ( this.m_texture.y < -halfTexSizeRad )
				this.m_texture.y += halfTexSize;
		}
		
		/*******************************************************************************************/
		
		// called when we've reached the end
		override protected function _onFinish():Boolean 
		{
			// check the position of the planet itself to see if we hit it
			var pp:Point 	= this.localToGlobal( new Point );
			var dist:Number	= MathHelper.dist2( pp.x, pp.y, this.m_ship.x, this.m_ship.y );
			if ( dist <= this.m_radius * this.scale )
			{
				this.signalOnHit.dispatch( this );
				return false; // we hit the planet, so don't reset
			}
			
			// check the position of the capture circle
			var cp:Point 	= this.m_captureOutline.localToGlobal( new Point( this.m_radius + this.m_captureSuppDist + this.m_captureRadius, 0.0 ) );
			dist			= MathHelper.dist2( cp.x, cp.y, this.m_ship.x, this.m_ship.y );
			if ( dist <= ( this.m_captureRadius * this.scale ) )
				this.signalOnCapture.dispatch( this );
			else
				this.signalOnMiss.dispatch( this, ( dist - ( this.m_captureRadius * this.scale ) ) );
				
			return true;
		}
		
		/*******************************************************************************************/
		
		// creates the displacement filter
		private function _createFilter():void
		{
			// check for an existing one
			var filter:DisplacementMapFilter = ( this.m_texture.filters.length > 0 ) ? this.m_texture.filters[0] : null;
			if ( filter != null )
				filter.mapBitmap.dispose();
			
			// create our new one
			var channel1:int 	= 1;
			var channel2:int 	= 2;
			var xMult:Number	= this.m_radius * this.scaleX;
			var yMult:Number	= this.m_radius * this.scaleY;
			var offset:Point	= new Point;
			var scale:Number	= ( this.m_radius * 2.0 ) / Planet.M_DISPLACE_SIZE;
			var bmd:BitmapData	= new BitmapData( Planet.M_DISPLACE_SIZE * scale * this.scaleX, Planet.M_DISPLACE_SIZE * scale * this.scaleY );
			filter				= new DisplacementMapFilter( bmd, offset, channel1, channel2, xMult, yMult, DisplacementMapFilterMode.COLOR );
			var m:Matrix		= new Matrix;
			m.scale( scale * this.scaleX, scale * this.scaleY );
			bmd.draw( this.m_displace, m );
			this.m_texture.filters = [filter];
		}
		
	}

}