package com.divillysausages.connected.logic 
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import flash.display.Shape;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class Streak extends Shape implements IUpdateObj, ISortable
	{
		
		/*******************************************************************************************/
		
		public var isPaused:Boolean = false;
		
		/*******************************************************************************************/
		
		private var m_ship:Ship 		= null;
		private var m_lineWidth:Number 	= 0.0;
		private var m_radians:Number 	= 0.0;
		private var m_inner:Number		= 0.0;
		private var m_outer:Number		= 0.0;
		
		/*******************************************************************************************/
		
		public function get active():Boolean { return ( this.parent != null && !isPaused ); }
		
		public function get zPos():Number { return -1.0; }
		
		/*******************************************************************************************/
		
		public function Streak( ship:Ship ) 
		{
			this.m_ship = ship;
			
			DS.addToUpdate( this );
			this.reset();
		}
		
		public function destroy():void
		{
			DS.removeFromUpdate( this );
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			this.m_ship = null;
		}
		
		public function reset():void
		{
			this.isPaused		= false;
			this.m_lineWidth 	= 2.0 + Math.random() * 4.0;
			this.m_radians 		= Math.random() * Math.PI * 2.0;
			this.m_inner		= 30 + Math.random() * 10.0;
			this.m_outer		= (( DS.stage.stageWidth > DS.stage.stageHeight ) ? DS.stage.stageWidth : DS.stage.stageHeight) + Math.random() * 200.0;
			this.x 				= this.m_ship.x;
			this.y 				= this.m_ship.y;
			this.alpha 			= 0.1 + Math.random() * 0.1;
			this._drawLine();
		}
		
		public function update( dt:Number ):void
		{
			// redraw our line so it follow the ship
			this._drawLine();
			
			this.x = this.m_ship.x;
			this.y = this.m_ship.y;
			
			this.alpha -= dt * 0.5;
			if ( this.alpha <= 0.0 )
				this.reset();
		}
		
		private function _drawLine():void
		{
			var pcX:Number = -this.m_ship.dir.x;
			var pcY:Number = 1.0 - ( ( this.m_ship.dir.y + 1.0 ) / 2.0 );
			
			var inX:Number = Math.cos( this.m_radians ) * this.m_inner;
			var inY:Number = Math.sin( this.m_radians ) * this.m_inner;
			var outX:Number	= inX + Math.cos( this.m_radians ) * this.m_outer + DS.stage.stageWidth * pcX;
			var outY:Number	= inY + Math.sin( this.m_radians ) * this.m_outer + DS.stage.stageHeight * pcY;
			
			this.graphics.clear();
			this.graphics.lineStyle( this.m_lineWidth, 0xffffff );
			this.graphics.moveTo( inX, inY );
			this.graphics.lineTo( outX, outY );
		}
		
	}

}