package com.divillysausages.connected.state 
{
	import com.divillysausages.connected.assets.Resources;
	import com.divillysausages.connected.constants.SoundConstants;
	import com.divillysausages.connected.logic.ISortable;
	import com.divillysausages.connected.logic.Particle;
	import com.divillysausages.connected.logic.Pic;
	import com.divillysausages.connected.logic.Planet;
	import com.divillysausages.connected.logic.ScreenFade;
	import com.divillysausages.connected.logic.ScreenFlash;
	import com.divillysausages.connected.logic.Ship;
	import com.divillysausages.connected.logic.Speech;
	import com.divillysausages.connected.logic.SpeechMsg;
	import com.divillysausages.connected.logic.Star;
	import com.divillysausages.connected.logic.Streak;
	import com.divillysausages.connected.state.constants.StateNames;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.fx.ScreenShaker;
	import com.divillysausages.ds.input.Input;
	import com.divillysausages.ds.input.Key;
	import flash.display.DisplayObject;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	
	/**
	 * The play state where we do our thing
	 * @author Damian Connolly
	 */
	public class StatePlay extends State
	{
		
		/*******************************************************************************************/
		
		public static var DID_DIE:Boolean = false;
		
		/*******************************************************************************************/
		
		private var m_ship:Ship 						= null; // the ship that we're using
		private var m_planets:Vector.<Planet>			= null; // all the planets that we're currently showing
		private var m_stars:Vector.<Star>				= null;	// the different stars that we have
		private var m_streaks:Vector.<Streak>			= null;	// the different streaks that we have for the player
		private var m_sortable:Vector.<ISortable>		= null; // all our sortable elements
		private var m_txtScore:TextField				= null;	// our score displayer
		private var m_screenFlash:ScreenFlash			= null;	// the screen flash for when we score
		private var m_screenShaker:ScreenShaker			= null;	// the screen shaker for when we miss
		private var m_screenFade:ScreenFade				= null;	// the screen fade for when we hit a planet
		private var m_pic:Pic							= null;	// the pic of the guy talking to us
		private var m_speech:Speech						= null;	// the speech for our guy
		private var m_score:int							= 0;	// our actual score
		private var m_misses:int						= 0;	// the number of misses
		private var m_speechText:Vector.<SpeechMsg>		= null;	// the different strings that we're going to start off with
		private var m_missTextAll:Vector.<SpeechMsg>	= null;	// the different strings for when we miss a planet (all)
		private var m_missText:Vector.<SpeechMsg>		= null;	// the different strings for when we miss a planet
		private var m_speechCount:int					= 0;	// where we are in the list of speech messages
		private var m_speechWaitTime:Number				= 0.0;	// the wait time for the speech
		private var m_speechCurrTime:Number				= 0.0;	// the current wait time for the speech
		private var m_hasPlayedTuto:Boolean				= false;// have we played the tutorial?
		private var m_numFuturePlanets:int				= 0;	// the number of future planets to come in
		
		/*******************************************************************************************/
		
		/**
		 * Creates our DS state
		 * @param fsm The Game finite state machine
		 */
		public function StatePlay( fsm:Game )
		{
			super( fsm, StateNames.STATE_PLAY );
			
			this.m_ship 		= new Ship;
			this.m_planets		= new Vector.<Planet>;
			this.m_stars		= new Vector.<Star>;
			this.m_streaks		= new Vector.<Streak>;
			this.m_sortable		= new Vector.<ISortable>;
			this.m_screenFlash	= new ScreenFlash( 0xffffff, 0.4 );
			this.m_screenShaker	= new ScreenShaker;
			this.m_screenFade	= new ScreenFade( DS.stage.color, 1.0 );
			this.m_screenFade.signalOnFinished.add( this._onScreenFadeFinish );
			
			// create our pic
			this.m_pic 		= new Pic;
			this.m_pic.x	= 10.0;
			this.m_pic.y	= 10.0;
			
			// create our speech
			this.m_speech 	= new Speech( 300.0 );
			this.m_speech.x = this.m_pic.x + this.m_pic.width + 10.0;
			this.m_speech.y = this.m_pic.y;
			
			// create our speech text
			this.m_speechText = new Vector.<SpeechMsg>;
			this.m_speechText.push( new SpeechMsg( "Welcome <SUBJECT_NAME>! Press SPACE if you can hear me.", -1.0 ) );
			this.m_speechText.push( new SpeechMsg( "You've been tasked with connecting planets to the intergalactic internet.", 5.0 ) );
			this.m_speechText.push( new SpeechMsg( "You will not* die.", 3.0 ) );
			this.m_speechText.push( new SpeechMsg( "Soon, billions of people will be connected in glorious 56kbps, and all through the latest in invisible cables...", 7.0 ) );
			this.m_speechText.push( new SpeechMsg( "Get used to the controls of your ship, and press SPACE when you're ready.", -1.0 ) );
			this.m_speechText.push( new SpeechMsg( "Start delivering!", 2.0 ) );
			this.m_speechText.push( new SpeechMsg( "Don't ruin this for me.", 2.5 ) );
			
			// create our fail text
			this.m_missText		= new Vector.<SpeechMsg>;
			this.m_missTextAll 	= new Vector.<SpeechMsg>;
			this.m_missTextAll.push( new SpeechMsg( "I'm sure they didn't want internet anyway.", 3.0 ) );
			this.m_missTextAll.push( new SpeechMsg( "Only a few billion people; don't worry, I'm sure they won't be missed.", 5.0 ) );
			this.m_missTextAll.push( new SpeechMsg( "Taking a joyride rather than doing your job? I hear ya.", 4.0 ) );
			this.m_missTextAll.push( new SpeechMsg( "They suck on that planet anyway.", 3.0 ) );
			this.m_missTextAll.push( new SpeechMsg( "Keep going, this is doing *wonders* for my numbers.", 4.0 ) );
			this.m_missTextAll.push( new SpeechMsg( "I'm going to get blamed for you missing that, you know.", 4.0 ) );
			this.m_missTextAll.push( new SpeechMsg( "You just don't care at all, do you?", 4.0 ) );
			this.m_missTextAll.push( new SpeechMsg( "*Literally* the complete opposite of what I asked you to do.", 5.0 ) );
			this.m_missTextAll.push( new SpeechMsg( "This is only costing us $1m a second, don't worry.", 4.5 ) );
			this.m_missTextAll.push( new SpeechMsg( "Good thing I'm a robot and not prone to murderous rage.", 5.0 ) );
			this.m_missTextAll.push( new SpeechMsg( "I guess my robot wife and robot baby will just go hungry this week.", 5.0 ) );
			this.m_missTextAll.push( new SpeechMsg( "Keep doing that; my robot claw is just hovering over the self-destruct button.", 5.0 ) );
			this.m_missTextAll.push( new SpeechMsg( "Remember when I said you weren't going to die? I lied.", 4.5 ) );
			
			// create our score
			var tf:TextFormat 					= new TextFormat( Resources.GAME_FONT, 40, 0xffffff );
			this.m_txtScore						= new TextField;
			this.m_txtScore.autoSize			= TextFieldAutoSize.LEFT;
			this.m_txtScore.defaultTextFormat	= tf;
			this.m_txtScore.multiline			= false;
			this.m_txtScore.wordWrap			= false;
			this.m_txtScore.mouseEnabled		= false;
			this.m_txtScore.mouseWheelEnabled	= false;
			this.m_txtScore.embedFonts			= true;
			this.m_txtScore.text				= "0";
			this.m_txtScore.x					= DS.stage.stageWidth - this.m_txtScore.width - 10;
			this.m_txtScore.y					= 10;
			this.m_txtScore.alpha				= 0.5;
		}
		
		override public function begin():void 
		{			
			// stars
			for ( var i:int = 0; i < 100; i++ )
				this._createStar();
			
			// streaks
			for ( i = 0; i < 20; i++ )
				this._createStreak();
			
			// make sure our ship is the foremost
			this.m_ship.x = DS.stage.stageWidth * 0.5;
			this.m_ship.y = DS.stage.stageHeight - 100.0;
			DS.canvas.addChild( this.m_ship );
			this._bringGUIToFront();
			
			CONFIG::debug { this.m_hasPlayedTuto = true; } // skip the stuff at the start
			
			// reset our score etc
			this.m_ship.isPaused	= false;
			this.m_score			= 0;
			this.m_misses			= 0;
			this.m_txtScore.text	= this.m_score + "";
			this.m_txtScore.x		= DS.stage.stageWidth - this.m_txtScore.width - 10;
			this.m_pic.show( 10.0, 10.0 );
			this.m_speechCount		= ( !this.m_hasPlayedTuto ) ? 0 : this.m_speechText.length;
			this.m_speechWaitTime	= 0.0;
			this.m_hasPlayedTuto	= true;
			this.m_numFuturePlanets	= 0;
			DS.callLater( this._showNextMessage, 300 );
			
			// add our click listener to show the text immediately
			DS.stage.addEventListener( MouseEvent.CLICK, this._showSpeechImmediately );
			
			DS.soundManager.playSoundtrack( Resources.MUSIC );
		}
		
		override public function end():void 
		{
			// destroy all our planets
			for each( var p:Planet in this.m_planets )
			{
				this.m_ship.onRemovePlanet( p );
				p.destroy();
			}
			this.m_planets.length = 0;
			
			// destroy all our stars
			for each( var s:Star in this.m_stars )
				s.destroy();
			this.m_stars.length = 0;
			
			// destroy all our streaks
			for each( var st:Streak in this.m_streaks )
				st.destroy();
			this.m_streaks.length = 0;
			
			// clear all particles
			Particle.clearParticles();
			
			// clear our speech
			this.m_speech.clear();
			
			// remove the other stuff
			this.m_ship.parent.removeChild( this.m_ship );
			this.m_txtScore.parent.removeChild( this.m_txtScore );
			this.m_pic.parent.removeChild( this.m_pic );
			this.m_speech.parent.removeChild( this.m_speech );
			
			// clear our listener
			DS.stage.addEventListener( MouseEvent.CLICK, this._showSpeechImmediately );
			Input.unbind( Key.SPACE );
			
			// clear our sortables
			this.m_sortable.length = 0;
			
			DS.soundManager.stopSound( Resources.MUSIC );
		}
		
		override public function update(dt:Number):void 
		{
			// go through and sort all our elements
			this.m_sortable.sort( this._sort );
			var len:int = this.m_sortable.length;
			for ( var i:int = 0; i < len; i++ )
			{
				var el:ISortable = this.m_sortable[i];
				el.parent.setChildIndex( ( el as DisplayObject ), i );
			}
			
			// add a ship particle
			if ( this.m_ship.parent != null )
				Particle.showParticle( this.m_ship, 0.8 + Math.random() );
				
			// update our speech text
			if ( this.m_speechCount <= this.m_speechText.length && this.m_speechWaitTime > 0.0 && this.m_speech.isFullyShown )
			{
				this.m_speechCurrTime += dt;
				if ( this.m_speechCurrTime >= this.m_speechWaitTime )
					this._showNextMessage();
			}
		}
		
		/*******************************************************************************************/
		
		private function _spaceToNextMessage():void
		{
			DS.soundManager.playSound( SoundConstants.SFX_SELECT, 0, 0.5 );
			this._showNextMessage();
		}
		
		// shows the next message in line
		private function _showNextMessage():void
		{
			// unbind any keys
			Input.unbind( Key.SPACE );
			
			// check our count index
			if ( this.m_speechCount >= this.m_speechText.length )
			{
				this.m_speech.hide();
				if ( this.m_planets.length == 0 )
					this._createNewPlanet(); // create our first planet
				return;
			}
			
			// show the next message
			var msg:SpeechMsg 		= this.m_speechText[this.m_speechCount++];
			this.m_speechWaitTime	= 0.0; // otherwise the showMsg won't work
			this._showMsg( msg.msg, msg.waitTime );
			if ( this.m_speechWaitTime <= 0.0 )
				Input.bind( Key.SPACE, this._spaceToNextMessage );
		}
		
		// shows a message
		private function _showMsg( msg:String, waitTime:Number ):void
		{
			if ( this.m_speechWaitTime == -1.0 ) // wait time is -1, so do nothing
				return;
			this.m_speechCurrTime 	= 0.0;
			this.m_speechWaitTime	= waitTime;
			this.m_speech.show( msg );
		}
		
		// show the speech now
		private function _showSpeechImmediately( e:MouseEvent = null ):void
		{
			this.m_speech.showImmediately();
		}
		
		// the sort function for our planets etc
		private function _sort( a:ISortable, b:ISortable ):int
		{
			return ( a.zPos > b.zPos ) ? -1 : ( a.zPos == b.zPos ) ? 0 : 1;
		}
		
		// called when we capture a planet
		private function _onCapturePlanet( p:Planet ):void
		{
			this.m_score++;
			this.m_txtScore.text 	= this.m_score + "";
			this.m_txtScore.x		= DS.stage.stageWidth - this.m_txtScore.width - 10;
			
			DS.soundManager.playSound( SoundConstants.SFX_SUCCESS );
			
			// add more planets (NOTE: it takes roughly about 3 seconds to get from one end to the other)
			if ( this.m_score >= 3 && this.m_numFuturePlanets < 2 )
			{
				this._showMsg( "You seem marginally competent, and in an amazing coincidence, there's now more planets", 7.0 );
				DS.callLater( this._createNewPlanet, 3000 );
				this.m_numFuturePlanets = 2;
			}
			else if ( this.m_score >= 10 && this.m_numFuturePlanets == 2 )
			{
				this._showMsg( "More planets! It's like this is a galaxy or something...", 5.0 );
				DS.callLater( this._createNewPlanet, ( ( p == this.m_planets[0] ) ? 6000 : 3000 ) );
				this.m_numFuturePlanets = 3;
			}
				
			// flash the screen
			this.m_screenFlash.show();
		}
		
		// fills the miss text
		private function _fillMissText():void
		{
			for each( var msg:SpeechMsg in this.m_missTextAll )
				this.m_missText.push( msg );
		}
		
		// called when we miss a planet
		private function _onMissPlanet( p:Planet, dist:Number ):void
		{
			this.m_misses++;
			if ( this.m_misses < 5 )
			{
				if ( this.m_missText.length == 0 )
					this._fillMissText();
				var msg:SpeechMsg = this.m_missText[int( Math.random() * this.m_missText.length )];
				this._showMsg( msg.msg, msg.waitTime );
			}
			else if ( this.m_misses == 5 )
				this._showMsg( "Miss one more moneypot and I'm going to blow you out of the sky. Beep boop.", -1 );
			else
			{
				this._onHitPlanet( p, false );
				return;
			}
			
			DS.soundManager.playSound( SoundConstants.SFX_MISS );
			
			// shake the screen
			var time:Number 		= 0.5;
			var moveStrength:Number	= 30.0;
			var rotStrength:Number	= 10.0;
			this.m_screenShaker.shake( time, moveStrength, rotStrength );
		}
		
		// called when we hit a planet
		private function _onHitPlanet( p:Planet, didDie:Boolean = true ):void
		{
			// pause everything
			this.m_ship.isPaused = true;
			for each( var p:Planet in this.m_planets )
				p.isPaused = true;
			for each( var s:Star in this.m_stars )
				s.isPaused = true;
			for each( var st:Streak in this.m_streaks )
				st.isPaused = true;
			Particle.pauseAllActiveParticle();
				
			// fade our screen
			this.m_screenFade.show();
			
			// shake the screen
			var time:Number 		= 1.0;
			var moveStrength:Number	= 30.0;
			var rotStrength:Number	= 10.0;
			this.m_screenShaker.shake( time, moveStrength, rotStrength );
			
			DS.soundManager.playSound( SoundConstants.SFX_EXPLOSION, 0, 0.3 );
			
			StatePlay.DID_DIE = didDie;
		}
		
		// creates a new planet
		private function _createNewPlanet():void
		{
			var p:Planet = new Planet( this.m_ship );
			DS.canvas.addChild( p );
			this._bringGUIToFront();
			
			// add our signals
			p.signalOnCapture.add( this._onCapturePlanet );
			p.signalOnMiss.add( this._onMissPlanet );
			p.signalOnHit.add( this._onHitPlanet );
			
			// tell the ship
			this.m_ship.onNewPlanet( p );
			
			// add it to our vectors
			this.m_planets.push( p );
			this.m_sortable.push( p );
		}
		
		// creates a new star
		private function _createStar():void
		{
			var star:Star = new Star( this.m_ship );
			DS.canvas.addChild( star );	
			this._bringGUIToFront();
			
			// add it to our vectors
			this.m_stars.push( star );
			this.m_sortable.push( star );
		}
		
		// creates a new streak
		private function _createStreak():void
		{
			var streak:Streak = new Streak( this.m_ship );
			DS.canvas.addChild( streak );
			
			// add it to our vector
			this.m_streaks.push( streak );
		}
		
		// make sure the gui is on front
		private function _bringGUIToFront():void
		{
			// add our score
			DS.canvas.addChild( this.m_txtScore );
			DS.canvas.addChild( this.m_pic );
			DS.canvas.addChild( this.m_speech );
		}
		
		// called when our screen fade has finished
		private function _onScreenFadeFinish():void
		{
			DS.callLater( this.m_screenFade.hide, 0 ); // next frame
			this.m_fsm.changeState( StateNames.STATE_GAME_OVER );
		}
		
	}

}